/*
 *
 * Pyldin-601 emulator version 2.1 for BeOS / Linux / Win9x/NT
 * Copyright (c) Sasha Chukov & Yura Kuznetsov, 2000-2003
 *
 */

#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <stdlib.h>

#include "mc6800.h"
#include "keyboard.h"
#include "timer.h"
#include "screen.h"
#include "floppy.h"

char *romName[] = {
	"str$08.rom",
	"str$09.rom",
	"str$0a.rom",
	"str$0b.rom",
	"str$0c.rom",
	"str$0d.rom",
	"str$0e.rom",
	"str$0f.rom"
};

word prn_c = 0xe634;
word prn_d = 0xe635;
byte e_printer = 0;
byte e_covox = 0xff;
byte lpt_ibm = 0;

int LoadRom(char *name, int p, mc6800 *pyl)
{
	char namp[9];
	namp[8] = 0;

	FILE *fi = fopen(name, "rb");
	if (fi) {
	  while ( fread(pyl->getRomPtr(p), 1, 8192, fi) == 8192 && p < 16) {
	    memcpy(namp, pyl->getRomPtr(p)+2, 8);
	    printf("%s -> Loaded %s rom as %d page\n", name, namp, p);
	  }
	  fclose(fi);
	}
	p++;
	return p;
}

void usage()
{
	printf(	"Usage:\n   pyldin [/?] [-A name] [-B name] [-L port] [/T] [/P] [/601A] [/260] [/29]\n"
		"      -A(-B) use disk(image) name (A: or B: for real floppy)\n"
		"      -L     set IBM printer port (1 - LPT1:, 2 - LPT2:, ...), DEFAULT - LPT1:\n"
		"      /601A  for emulation model Pyldin 601A\n"
		"      /T     set IBM date/time to Pyldin\n"
		"      /P     for enable printer emulation (this disables COVOX on 601)\n"
		"      /260   for emulation model of Pyldin 601 without hardware bug\n"
		"      /29    for view 28/29 lines on Pyldin 601 (for BIOS 2.60)\n\n"
		"Example: pyldin -A B: -B games.img\n"
		"Default: pyldin -A A: -B B: -L 1\n" );
	exit(0);
}

void main(int argc, char *argv[])
{
	byte *m;
	word ZwaitW,Zwait_W2;
	union REGS regs;
	char *dname0 = "A:", *dname1 = "B:", mas1[2], *b_name = "bios.rom";
	char *char_lpt;
	FILE *fi;
	byte set_DT = 0;

	printf(	"Pyldin-601/601A emulator v3.0  (c) Sasha Chukov & Yura Kuznetsov, 1999-2005\n"
		"for help type  pyldin /?  or view file readme.!!!\n\n" );

	byte i = 1;
	while (i < argc) {
		if ((strcmp(argv[i], "?") == 0) ||
			(strcmp(argv[i], "-?") == 0) ||
			(strcmp(argv[i], "/?") == 0) ||
			(strcmp(argv[i], "-H") == 0) ||
			(strcmp(argv[i], "-h") == 0)) usage();
		if ((strcmp(argv[i], "-A") == 0) ||
			(strcmp(argv[i], "-a") == 0)) dname0 = argv[++i];
		if ((strcmp(argv[i], "-B") == 0) ||
			(strcmp(argv[i], "-b") == 0)) dname1 = argv[++i];
		if ((strcmp(argv[i], "-L") == 0) ||
			(strcmp(argv[i], "-l") == 0)) {
				char_lpt = argv[++i];
				if (strcmp(char_lpt, "2") == 0) lpt_ibm = 1;
				if (strcmp(char_lpt, "3") == 0) lpt_ibm = 2;
				if (strcmp(char_lpt, "4") == 0) lpt_ibm = 3;
		}
		if (strcmp(argv[i], "/260") == 0) {
				txt260 = 1;
				grf260 = 0;
				hor_correct = 0;
				enable320 = 0xff;
		}
		if (strcmp(argv[i], "/29") == 0) Enable2829 = 0xff;
		if ((strcmp(argv[i], "/t") == 0) ||
			(strcmp(argv[i], "/T") == 0)) set_DT = 0xff;
		if ((strcmp(argv[i], "/p") == 0) ||
			(strcmp(argv[i], "/P") == 0)) {
				e_covox = 0;
				e_printer = 0xff;
		}
		if ((strcmp(argv[i], "/601a") == 0) ||
			(strcmp(argv[i], "/601A") == 0)) {
				m601a = 0xff;
				prn_c = 0xe630;
				prn_d = 0xe631;
		}
		i++;
	}

	if (m601a == 0xff) e_covox = 0xff;

	mc6800 *pyl = new mc6800();

	// ⠥ ﭨ ⥫  㪠
	fi = fopen("wait_cnt.cpu", "rb");
	if (fi) {
		fread(mas1,1,3,fi);
		Zwait=mas1[0] | mas1[1]<<8;
		fSound=mas1[2];
		fclose(fi);
	}

	// 㧨   BIOS
	if (m601a != 0) b_name="bios_a.rom";
	printf("Loading main rom %s     ... ",b_name);
	fi = fopen(b_name, "rb");
	if (fi) {
		m = pyl->getRomPtr(-1); fread(m, 1, 4096, fi);
		fclose(fi); printf("Ok\n");
	} else {
		printf("\07Failed!\n");
		delete pyl;
		exit(-1);
	}

	// 㧨 䮭 ⥪⮢ ०
	printf("Loading font rom video.rom    ... ");
	if (loadTextFont("video.rom") == 0) printf("Ok\n");
	else {
		printf("\07Failed!\n");
		delete pyl;
		exit(-1);
	}

	int j = 8;
	// ⠥ 㧨 8 襪 ࠭筮 
	for (i = 0; i < 8; i++)
		j = LoadRom(romName[i], j, pyl);

	floppyOn();	// 㡠 䫮  襣 ꫤ

	// 㧨  A:
	printf("Loading disk image A:      ... ");
	if (loadDiskImage(dname0, FLOPPY_A) == 0) printf("Ok\n");
	else printf("Failed! Use floppy A:\n");

	// 㧨  B:
	printf("Loading disk image B:      ... ");
	if (loadDiskImage(dname1, FLOPPY_B) == 0) printf("Ok\n");
	else printf("Failed! Use floppy B:\n");

	if (e_printer != 0) {
		regs.h.ah=1; regs.w.dx=0; int386(0x17,&regs,&regs); //ਭ
	}

	// ⮡ 뫮 㤮   
	regs.w.ax=0x0305; regs.w.bx=0;
	int386(0x16,&regs,&regs);

	printf("Press Any Key for start !\n");
	getch();

	regs.w.ax=0x12;			// 樠㥬 
	int386(0x10,&regs,&regs);
	outp(0x3ce,5); outp(0x3cf,0);
	outp(0x3ce,8); outp(0x3cf,0xff);

	_disable();
	outp(0x43, 0x36);
	outp(0x40, 0x30);
	outp(0x40, 0x56);
	_enable();

	installTimerHandler();		// ⠫ ⠩
	installKeyboardHandler();	// ⠫ 

	init765();
	viewInfo();

	if (set_DT) pyl->setDATETIME();

	pyl->reset();			//  
	do {
		if (fRef > 6) {
			if (m601a == 0) refreshScr(); else refreshScr_A();
			fRef = 0;
		}

		pyl->step();	// 믮塞  

// ⥫
		ZwaitW=Zwait;
		for (;ZwaitW>0;ZwaitW--) {
			Zwait_W2=9;
			do {
				Zwait_W2--;
			} while (Zwait_W2 != 0);
		}

		if (resetRequested == 1) {
			pyl->reset();
			resetRequested = 0;
		}

	} while (exitRequested == 0);	// 横   室

	removeKeyboardHandler();		// . 
	removeTimerHandler();			// . ⠩

	_disable();
	outp(0x43, 0x36);
	outp(0x40, 0x00);
	outp(0x40, 0x00);
	_enable();

	regs.w.ax=0x03;			// ⥪⮢ ࠭ 80x25
	int386(0x10,&regs,&regs);

	init765();

	fi = fopen("wait_cnt.cpu", "wb");
	if (fi) {
		mas1[0]=Zwait; mas1[1]=Zwait>>8;
		mas1[2]=fSound;
		fwrite(mas1,1,3,fi);
		fclose(fi);
	}

	// ࠭ ᪮ 
	unloadDiskImage(dname0, FLOPPY_A);
	unloadDiskImage(dname1, FLOPPY_B);

	floppyOff();	// 몫 䫮 ⥬

	delete pyl;
}
